﻿<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:msxsl="urn:schemas-microsoft-com:xslt"
                xmlns:ext="http://exslt.org/common"
                exclude-result-prefixes="ext msxsl"
  >

    <xsl:output method="xml" indent="no"/>

    <!-- Глобальные переменные -->
    <xsl:variable name ="TransactionBlock">TransactionBlock</xsl:variable>
    <xsl:variable name ="FunctionNumber">FunctionNumber</xsl:variable>
    <xsl:variable name ="TransactionTimestamp">TransactionTimestamp</xsl:variable>
    <xsl:variable name ="TransactionId">TransactionId</xsl:variable>
    <xsl:variable name ="TerminalId">TerminalId</xsl:variable>

    <!-- Список функций для конвертации -->
    <xsl:param name="FunctionMap">
        <Function name="ACCOUNT PAYMENT">1</Function>
        <Function name="AUTHORISATION FAILED">2</Function>
        <Function name="AUTHORISATION SUCCESSFUL">3</Function>
        <Function name="BANK SERVICES BALANCE ENQUIRY">4</Function>
        <Function name="BANK SERVICES DEPOSIT">5</Function>
        <Function name="BANK SERVICES MINI STATEMENT">6</Function>
        <Function name="BANK SERVICES WITHDRAWAL">7</Function>
        <Function name="BLIND SPOT CHECK">8</Function>
        <Function name="CHANGE">9</Function>
        <Function name="DEPARTMENT SALE">10</Function>
        <Function name="ENQUIRY BOND RATES">11</Function>
        <Function name="ENQUIRY PRICE">12</Function>
        <Function name="ENQUIRY GIFT CARD BALANCE">13</Function>
        <Function name="ITEM DISCOUNT">14</Function>
        <Function name="ITEM PRICE OVERRIDE">15</Function>
        <Function name="ITEM REFUND">16</Function>
        <Function name="ITEM SALE">17</Function>
        <Function name="ITEM UNKNOWN">18</Function>
        <Function name="ITEM VOID">19</Function>
        <Function name="LOGOFF">20</Function>
        <Function name="LOGON">21</Function>
        <Function name="LOGON INVALID">22</Function>
        <Function name="MANAGER MENU">23</Function>
        <Function name="MONEY TRANSFER DEPOSIT">24</Function>
        <Function name="MONEY TRANSFER WITHDRAWAL">25</Function>
        <Function name="NO SALE">26</Function>
        <Function name="PASSWORD CHANGE">27</Function>
        <Function name="PASSWORD FORCE CHANGE">28</Function>
        <Function name="REMOVE CASH DRAWER">29</Function>
        <Function name="RESTART">30</Function>
        <Function name="SECURE">31</Function>
        <Function name="SECURE AUTO">32</Function>
        <Function name="SECURE OVERRIDE">33</Function>
        <Function name="SESSION OVERRIDE">34</Function>
        <Function name="TENDER">35</Function>
        <Function name="TENDER VOID">36</Function>
        <Function name="TRAINING MODE ENTER">37</Function>
        <Function name="TRAINING MODE EXIT">38</Function>
        <Function name="TRANSACTION COMPLETE">39</Function>
        <Function name="TRANSACTION DISCOUNT">40</Function>
        <Function name="TRANSACTION LOYALTY">41</Function>
        <Function name="TRANSACTION RECALL">42</Function>
        <Function name="TRANSACTION REPRINT">43</Function>
        <Function name="TRANSACTION SEARCH">44</Function>
        <Function name="TRANSACTION SUSPEND">45</Function>
        <Function name="TRANSACTION TOTAL">46</Function>
        <Function name="TRANSACTION VOID">47</Function>
        <Function name="OPEN CASH DRAWER">48</Function>
        <Function name="VAS FAILED">49</Function>
        <Function name="SECURE UNLOCK">50</Function>
    </xsl:param>

    <!-- Основная функция трансформации -->
    <xsl:template match="POSEventMessage">
        <xsl:element name="{$TransactionBlock}">
            <xsl:element name="{$TransactionTimestamp}">
                <xsl:value-of select="./date"/>
                <xsl:text disable-output-escaping="yes"><![CDATA[ ]]></xsl:text>
                <xsl:value-of select="./time"/>
            </xsl:element>
            <xsl:element name ="{$TerminalId}">
                <xsl:value-of select="./till"/>
            </xsl:element>
            <xsl:for-each select="*">
                <xsl:choose>
                    <xsl:when test="name() = 'date'"/>
                    <xsl:when test="name() = 'time'"/>
                    <xsl:when test ="name() = 'txnNumber'">
                        <xsl:element name="{$TransactionId}">
                            <xsl:value-of select="."/>
                        </xsl:element>
                    </xsl:when>
                    <xsl:when test="name() = 'event'">
                        <xsl:call-template name="eventTranslate">
                            <xsl:with-param name="node" select="."/>
                        </xsl:call-template>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:call-template name="nodeTranslate"/>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:for-each>
        </xsl:element>
    </xsl:template>
    
    <!-- Конвертация элемента события-->
    <xsl:template name="eventTranslate">
        <xsl:param name="node"/>
        <xsl:element name="{$FunctionNumber}">
            <xsl:variable name="nameFunction" select="$node/@type"/>
            <xsl:value-of select="ext:node-set($FunctionMap)/Function[@name=$nameFunction]"/>
        </xsl:element>
        <xsl:for-each select="$node/*">
            <xsl:call-template name="nodeTranslate"/>
        </xsl:for-each>
    </xsl:template>

    <!--Конвертация элемента-->
    <xsl:template name ="nodeTranslate">
        <xsl:variable name="nodeName" select="name()"/>
        <xsl:call-template name="attributeTranslate"/>
        <xsl:call-template name="childrenTranslate"/>
        <xsl:if test="not(*)">
            <xsl:element name="{name()}">
                <xsl:value-of select="."/>
            </xsl:element>
        </xsl:if>
    </xsl:template>

    <!--Конвертация дочерних элементов-->
    <xsl:template name="childrenTranslate">
        <xsl:variable name="nodeName" select="name()"/>
        <xsl:for-each select="*">
            <xsl:variable name="childrenName" select="name()"/>
            <xsl:element name="{$nodeName}_{$childrenName}">
                <xsl:value-of select="."/>
            </xsl:element>
        </xsl:for-each>
    </xsl:template>

    <!-- Конвертация аттрибут -->
    <xsl:template name="attributeTranslate">
        <xsl:variable name="nodeName" select="name()"/>
        <xsl:for-each select="@*">
            <xsl:element name="{$nodeName}_{name()}">
                <xsl:value-of select="."/>
            </xsl:element>
        </xsl:for-each>
    </xsl:template>

</xsl:stylesheet>
